/*
   This program is similar to example1.c. It lets you switch between
   perspective and parallel projections and it lets you move the
   camera around (using sliders this time).

   The main difference is that this program shows a much larger field
   of view and it draws a grid on the horizontal xz-plane so that you
   can see how, in perspective projection, parallel lines converge
   towards a point at infinity.
*/
#include <windows.h>
#include "glut.h"
#include "glui.h"

/* GLUT callback function prototypes */
void display(void);
void reshape(int, int);
void slider1(double);
void slider2(double);
void slider3(double);
void keyboard(unsigned char, int, int);

int windowHeight;         // this is set by the glut reshape() callback function
int windowWidth;          // this is set by the glut reshape() callback function

int perspective = 1; /* choose between perspective and parallel projections */

#define sliderInit 0.5
double slider1Value = sliderInit;  // this is set by the glui slider() callback function
double slider2Value = sliderInit;
double slider3Value = sliderInit;

int main(int argc, char** argv)
{
   int window;
   /* GLUT initialization */
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB); // glui needs double buffering!
   glutInitWindowSize(500,560);       // width, height (in pixels)
   glutInitWindowPosition(100,150);   // left, top (in pixels)
   window = glutCreateWindow(argv[0]);
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 10,  0, -10, 20, sliderInit, slider1);
   gluiHorizontalSlider(window, 10, 20, -10, 20, sliderInit, slider2);
   gluiHorizontalSlider(window, 10, 40, -10, 20, sliderInit, slider3);

   /* Initialize OpenGL state */
   glShadeModel(GL_FLAT);          /* default is GL_SMOOTH */
   glFrontFace(GL_CCW);
   glPolygonMode(GL_FRONT, GL_LINE);
   glPolygonMode(GL_BACK,  GL_LINE);
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */

   glutMainLoop(); /* Enter event loop */
   return 0;       /* We never get here */
}//main()


void frame(float gray)
{
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glColor3f(gray, gray, gray);  // Draw in gray.
   glBegin(GL_LINES);            // Draw a set of axes.
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(1.2, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 1.2, 0.0);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 1.2);
   glEnd();
   glPopAttrib();
}//frame()


void grid(int n, int m)
{
   int i;
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glColor3f(0.7, 0.7, 0.7);  // Draw in light gray.
   // Draw an n by m coordinate grid.
   glBegin(GL_LINES);
     for (i = -n; i <= n; i++)
     {
        glVertex3f(-m, 0.0,  i);
        glVertex3f( m, 0.0,  i);
     }
     for (i = -m; i <= m; i++)
     {
        glVertex3f( i, 0.0, -n);
        glVertex3f( i, 0.0,  n);
     }
   glEnd();
   glPopAttrib();
}

/* The display() callback function */
void display( void )
{
   float left, right, bottom, top, zNear, zFar;
   float eyex, eyey, eyez, centerx, centery, centerz, upx, upy, upz;

   glClear(GL_COLOR_BUFFER_BIT);

   // specify the viewport
   glViewport( 0,  0, windowWidth, windowHeight-60); // left, bottom, width, height (in pixels)

   // set up viewing
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // parameters for glFrustum() or glOrtho()
   left = -3.0;
   right = 3.0;
   bottom = -3.0;
   top = 3.0;
   zNear = 2.0;
   zFar = 100.0;  // a very deep field of view
   if (perspective)
   {
      glFrustum(left, right, bottom, top, zNear, zFar);
   }
   else /* use parallel projection */
   {
      glOrtho  (left, right, bottom, top, zNear, zFar);
   }


   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   eyex = 10.0*slider1Value;
   eyey = 10.0*slider2Value;
   eyez = 10.0*slider3Value;
   centerx = 0.0;
   centery = 0.0;
   centerz = 0.0;
   upx = 0.0;
   upy = 1.0;
   upz = 0.0;
   gluLookAt(eyex, eyey, eyez, centerx, centery, centerz, upx, upy, upz);

   grid(50,100);        // Show a horizontal grid in xz-plane.

   frame(0.0);         // Show a coordinate frame.

   glColor3f(0.0, 0.0, 1.0);  // Draw in blue
   glutWireCube(1.0);

   printf("eyex = %f, eyey = %f, eyez = %f\n", eyex, eyey, eyez);

   glutSwapBuffers();  // glui needs double buffering!
}//display()


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback functions */
void slider1(double value)
{
   slider1Value = value;

   //printf("We got a \"slider1(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider1()

void slider2(double value)
{
   slider2Value = value;

   //printf("We got a \"slider2(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider2()

void slider3(double value)
{
   slider3Value = value;

   //printf("We got a \"slider3(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider3()


/* The keyboard() callback function */
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 'o':
         perspective = 0;
         break;
      case 'O':
         perspective = 0;
         break;
      case 'p':
         perspective = 1;
         break;
      case 'P':
         perspective = 1;
         break;
      default:
         break;
   }

   // Notice that keyboard input is not buffered
   // (like it is in the C Library).
   //printf("We got a \"keyboard(%c, %d, %d)\" event\n", key, x, y);

   glutPostRedisplay();
}//keyboard()
