/*
   In this program, the slider causes a cube to move away from the "camera".
   (When the slider is all the way to the left, the center of the cube is
   three units away from the camera. When the slider is all the way to the
   right, the center of the cube is 53 units away from the camera.)

   As the cube moves away, the front plane of the view frustum moves away from
   the camera along with the cube (and the size of the front plane remains fixed).

   The result is that the cube appears to grow larger as it moves away!
   The reason is that, as the front plane moves away from the camera, the
   perspective projection becomes more and more like a parallel (orthogonal)
   projection.

   Notice that this program shows that thinking of the "center of projection" (COP)
   as an "eye" or "camera" can be misleading. When you move an eye (or a camera)
   away from what it is looking at, the object will always appear smaller, never
   larger. But the distance between the COP and an object is only one (of many)
   parameters in determining how OpenGL renders the object. You need to consider
   all of these parameters to know exactly how the object will get rendered.
*/
#include <windows.h>
#include "glut.h"
#include "glui.h"

/* GLUT callback function prototypes */
void display(void);
void reshape(int, int);
void slider(double);
void keyboard(unsigned char, int, int);

int windowHeight;         // this is set by the glut reshape() callback function
int windowWidth;          // this is set by the glut reshape() callback function

int viewChoice = 1;

#define sliderInit 0.0
double sliderValue = sliderInit;  // this is set by the glui slider() callback function

int main(int argc, char** argv)
{
   int window;
   /* GLUT initialization */
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB); // glui needs double buffering!
   glutInitWindowSize(500,520);       // width, height (in pixels)
   glutInitWindowPosition(100,150);   // left, top (in pixels)
   window = glutCreateWindow(argv[0]);
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 10,  0, -10, 20, sliderInit, slider);

   /* Initialize OpenGL state */
   glShadeModel(GL_FLAT);          /* default is GL_SMOOTH */
   glFrontFace(GL_CCW);
   glPolygonMode(GL_FRONT, GL_LINE);
   glPolygonMode(GL_BACK,  GL_LINE);
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */

   glutMainLoop(); /* Enter event loop */
   return 0;       /* We never get here */
}//main()


/* The display() callback function */
void display( void )
{
   double left, right, bottom, top, zNear, zFar;
   double centerOfCube = 3.0+sliderValue*50.0;

   glClear(GL_COLOR_BUFFER_BIT);

   // specify the viewport
   glViewport( 0,  0, windowWidth, windowHeight-20); // left, bottom, width, height (in pixels)

   // specify the view volume
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   left   = -2.0;   // parameters for glFrustum() or glOrtho()
   right  =  2.0;
   bottom = -2.0;
   top    =  2.0;
   zNear = centerOfCube - 2.0;
   zFar  = centerOfCube + 2.0;
   if (viewChoice == 0)
   {
      glOrtho  (left, right, bottom, top, zNear, zFar);
   }
   else
   {
      glFrustum(left, right, bottom, top, zNear, zFar);
   }

   // draw the model (inside of the view volume)
   glColor3f(0.0, 0.0, 1.0);    // sraw in blue
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glTranslated(0.0, 0.0, -centerOfCube);
   glutWireCube(2.0);

   glutSwapBuffers();  // glui needs double buffering!
}//display()


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback functions */
void slider(double value)
{
   sliderValue = value;

   printf("We got a \"slider(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider()


/* The keyboard() callback function */
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 'o':
         viewChoice = 0;
         break;
      case 'O':
         viewChoice = 0;
         break;
      case 'p':
         viewChoice = 1;
         break;
      case 'P':
         viewChoice = 1;
         break;
      default:
         break;
   }

   // Notice that keyboard input is not buffered
   // (like it is in the C Library).
   printf("We got a \"keyboard(%c, %d, %d)\" event\n", key, x, y);

   glutPostRedisplay();
}//keyboard()
